// $Id: CompactCompilerFactory.cpp,v 1.2 2007/03/17 19:19:26 david Exp $ -*- c++ -*-
/*
 * $Log: CompactCompilerFactory.cpp,v $
 * Revision 1.2  2007/03/17 19:19:26  david
 * *** empty log message ***
 *
 * Revision 1.1  2007/03/14 01:20:19  david
 * Changed compiler implementation to use evaluation strategy objects
 * instead of functions. These strategies are selected according to the
 * target selected from the command line options or from clues such as
 * the output file extension.
 *
 * Revision 1.1  2007/03/13 00:26:20  david
 * This version of the Compact compiler uses the new CDK2 library.
 * All nodes and visitors now use non-overloaded visitor hooks. The Compact
 * compiler now implements the abstract factory for scanners and compilers.
 */

#include <iostream>

#include "CompactCompilerFactory.h"
#include "CompactScanner.h"
#include "CompilerImpl.h"

/**
 * This object is automatically registered by the constructor in the
 * superclass' language registry.
 */
CompactCompilerFactory CompactCompilerFactory::_thisFactory("compact");

/**
 * Create a scanner object for the Compact language.
 * This method is now private and is called only from the compiler creation
 * method.
 * @param name name of the input file (for debug only)
 * @return scanner object pointer
 * @see createCompiler
 */
FlexLexer *CompactCompilerFactory::createScanner(const char *name) {
  return new CompactScanner(name, NULL, NULL);
}

/**
 * Create a compiler object for a given language.
 * @param name name of the language handled by the compiler
 * @return compiler object pointer
 */
cdk::Compiler *CompactCompilerFactory::createCompiler(const char *name) {
  FlexLexer *scanner = createScanner(name);
  return new CompilerImpl(name, scanner);
}
