// $Id: CompactCompilerFactory.h,v 1.2 2007/03/17 19:19:26 david Exp $ -*- c++ -*-
/*
 * $Log: CompactCompilerFactory.h,v $
 * Revision 1.2  2007/03/17 19:19:26  david
 * *** empty log message ***
 *
 * Revision 1.1  2007/03/14 01:20:20  david
 * Changed compiler implementation to use evaluation strategy objects
 * instead of functions. These strategies are selected according to the
 * target selected from the command line options or from clues such as
 * the output file extension.
 *
 * Revision 1.1  2007/03/13 00:26:20  david
 * This version of the Compact compiler uses the new CDK2 library.
 * All nodes and visitors now use non-overloaded visitor hooks. The Compact
 * compiler now implements the abstract factory for scanners and compilers.
 */
#ifndef __COMPACT_COMPACTCOMPILERFACTORY_H__
#define __COMPACT_COMPACTCOMPILERFACTORY_H__

#include <cdk/CompilerFactory.h>

class FlexLexer;
namespace cdk { class Compiler; }

/**
 * This class implements the compiler factory for the Compact compiler.
 */
class CompactCompilerFactory : public cdk::CompilerFactory {
  /**
   * This object is automatically registered by the constructor in the
   * superclass' language registry.
   */
  static CompactCompilerFactory _thisFactory;

protected:
  /**
   * @param language name of the language handled by this factory (see .cpp file)
   */
  CompactCompilerFactory(const char *language) : cdk::CompilerFactory(language) {}

protected:
  /**
   * Create a scanner object for the Compact language.
   * This method is now private and is called only from the compiler creation
   * method.
   * @param name name of the input file (for debug only)
   * @return scanner object pointer
   * @see createCompiler
   */
  virtual FlexLexer *createScanner(const char *name);

public:
  /**
   * Create a compiler object for a given language.
   * @param name name of the language handled by the compiler
   * @return compiler object pointer
   */
  virtual cdk::Compiler *createCompiler(const char *name);

}; // class CompactCompilerFactory

#endif
