%{
/* $Id: CompactParser.y,v 1.8 2008/03/01 18:32:48 david Exp $ */
/*
 * $Log: CompactParser.y,v $
 * Revision 1.8  2008/03/01 18:32:48  david
 * Compact for CDK3 (first version).
 *
 * Revision 1.7  2007/03/17 19:19:27  david
 * *** empty log message ***
 *
 * Revision 1.6  2007/03/14 01:20:20  david
 * Changed compiler implementation to use evaluation strategy objects
 * instead of functions. These strategies are selected according to the
 * target selected from the command line options or from clues such as
 * the output file extension.
 *
 * Revision 1.5  2007/03/13 00:26:19  david
 * This version of the Compact compiler uses the new CDK2 library.
 * All nodes and visitors now use non-overloaded visitor hooks. The Compact
 * compiler now implements the abstract factory for scanners and compilers.
 *
 * Revision 1.4  2006/03/19 21:46:50  david
 * Miscellaneous corrections to i/o streams.
 *
 * Revision 1.2  2006/01/03 18:10:46  david
 * Evaluators are now defined separately from the parser.
 */
#include <iostream>
#include <string>

#include "CompilerImpl.h"

#include <cdk/nodes/Node.h>
#include <cdk/nodes/Nil.h>
#include <cdk/nodes/Sequence.h>

#include <cdk/nodes/expressions/Expression.h>
#include <cdk/nodes/expressions/Integer.h>
#include <cdk/nodes/expressions/Double.h>
#include <cdk/nodes/expressions/String.h>
#include <cdk/nodes/expressions/Identifier.h>
#include <cdk/nodes/expressions/NEG.h>
#include <cdk/nodes/expressions/ADD.h>
#include <cdk/nodes/expressions/SUB.h>
#include <cdk/nodes/expressions/MUL.h>
#include <cdk/nodes/expressions/DIV.h>
#include <cdk/nodes/expressions/MOD.h>
#include <cdk/nodes/expressions/LT.h>
#include <cdk/nodes/expressions/GT.h>
#include <cdk/nodes/expressions/GE.h>
#include <cdk/nodes/expressions/LE.h>
#include <cdk/nodes/expressions/NE.h>
#include <cdk/nodes/expressions/EQ.h>

#include "nodes/ReadNode.h"
#include "nodes/PrintNode.h"
#include "nodes/AssignmentNode.h"
#include "nodes/WhileNode.h"
#include "nodes/IfNode.h"
#include "nodes/IfElseNode.h"
#include "nodes/ProgramNode.h"

#define LINE scanner()->lineno()
#define yylex scanner()->yylex

// The rules below will be included in the function yyparse, the main
// parsing function. The macro below will cause the function to become
// a method instead. This is very fragile, but works with byacc.
// The class is CompilerImpl, an implementation of cdk::Compiler.
#define yyparse CompilerImpl::yyparse

%}

%union {
  int                                i;	/* integer value */
  std::string                       *s;	/* symbol name or string literal */
  cdk::node::Node                   *node;	/* node pointer */
  cdk::node::Sequence               *sequence;
  cdk::node::expression::Expression *expression; /* expression nodes */
};

%token <i> CPT_INTEGER
%token <s> CPT_VARIABLE CPT_STRING
%token WHILE IF PRINT READ PROGRAM END
%nonassoc IFX
%nonassoc ELSE

%left CPT_GE CPT_LE CPT_EQ CPT_NE '>' '<'
%left '+' '-'
%left '*' '/' '%'
%nonassoc UMINUS

%type <node> stmt program
%type <sequence> list
%type <expression> expr

%%

program	: PROGRAM list END     { _syntax = new ProgramNode(LINE, $2); }
	;

stmt	: ';'                  { $$ = new cdk::node::Nil(LINE); }
	| PRINT CPT_STRING ';' { $$ = new cdk::node::expression::String(LINE, $2); }
	| PRINT expr ';'       { $$ = new PrintNode(LINE, $2); }
	| READ CPT_VARIABLE ';'
        {
          $$ = new ReadNode(LINE, new cdk::node::expression::Identifier(LINE, $2));
        }
	| CPT_VARIABLE '=' expr ';'
        {
          $$ = new AssignmentNode(LINE, new cdk::node::expression::Identifier(LINE, $1), $3);
        }
	| WHILE '(' expr ')' stmt        { $$ = new WhileNode(LINE, $3, $5); }
	| IF '(' expr ')' stmt %prec IFX { $$ = new IfNode(LINE, $3, $5); }
	| IF '(' expr ')' stmt ELSE stmt { $$ = new IfElseNode(LINE, $3, $5, $7); }
	| '{' list '}'                   { $$ = $2; }
	;

list	: stmt	    { $$ = new cdk::node::Sequence(LINE, $1); }
	| list stmt { $$ = new cdk::node::Sequence(LINE, $2, $1); }
	;

expr : CPT_INTEGER             { $$ = new cdk::node::expression::Integer(LINE, $1); }
     | CPT_VARIABLE            { $$ = new cdk::node::expression::Identifier(LINE, $1); }
     | '-' expr %prec UMINUS   { $$ = new cdk::node::expression::NEG(LINE, $2); }
     | expr '+' expr	       { $$ = new cdk::node::expression::ADD(LINE, $1, $3); }
     | expr '-' expr	       { $$ = new cdk::node::expression::SUB(LINE, $1, $3); }
     | expr '*' expr	       { $$ = new cdk::node::expression::MUL(LINE, $1, $3); }
     | expr '/' expr	       { $$ = new cdk::node::expression::DIV(LINE, $1, $3); }
     | expr '%' expr	       { $$ = new cdk::node::expression::MOD(LINE, $1, $3); }
     | expr '<' expr	       { $$ = new cdk::node::expression::LT(LINE, $1, $3); }
     | expr '>' expr	       { $$ = new cdk::node::expression::GT(LINE, $1, $3); }
     | expr CPT_GE expr	       { $$ = new cdk::node::expression::GE(LINE, $1, $3); }
     | expr CPT_LE expr        { $$ = new cdk::node::expression::LE(LINE, $1, $3); }
     | expr CPT_NE expr	       { $$ = new cdk::node::expression::NE(LINE, $1, $3); }
     | expr CPT_EQ expr	       { $$ = new cdk::node::expression::EQ(LINE, $1, $3); }
     | '(' expr ')'            { $$ = $2; }
     ;

%%
