%option stack noyywrap yylineno 8bit debug
%option c++ yyclass="CompactScanner" outfile="CompactScanner.cpp"
%{ 
/* $Id: CompactScanner.l,v 1.6 2008/03/01 18:32:48 david Exp $ */
/*
 * $Log: CompactScanner.l,v $
 * Revision 1.6  2008/03/01 18:32:48  david
 * Compact for CDK3 (first version).
 *
 * Revision 1.5  2007/03/17 19:19:27  david
 * *** empty log message ***
 *
 * Revision 1.4  2007/03/14 01:20:20  david
 * Changed compiler implementation to use evaluation strategy objects
 * instead of functions. These strategies are selected according to the
 * target selected from the command line options or from clues such as
 * the output file extension.
 *
 * Revision 1.3  2007/03/13 00:26:19  david
 * This version of the Compact compiler uses the new CDK2 library.
 * All nodes and visitors now use non-overloaded visitor hooks. The Compact
 * compiler now implements the abstract factory for scanners and compilers.
 *
 * Revision 1.2  2006/04/03 18:11:38  david
 * Added debug settings to flex files.
 */
#include <string>
#include <cdk/nodes/Node.h>
#include <cdk/nodes/Sequence.h>
#include <cdk/nodes/expressions/Expression.h>
#include "CompactScanner.h"
// ensure that relevant includes are made before including the following
#include "CompactParser.tab.h"
%}
%%
"#".*			; /* comment */
">="			return CPT_GE; 
"<="			return CPT_LE; 
"=="			return CPT_EQ; 
"!="			return CPT_NE; 
"while"			return WHILE; 
"if"			return IF; 
"else"			return ELSE; 
"print"			return PRINT; 
"read"			return READ; 
"program"		return PROGRAM; 
"end"			return END; 

[A-Za-z][A-Za-z0-9_]*	{ yylval.s = new std::string(yytext); return CPT_VARIABLE; }
\'[^']*\'		{ yytext[yyleng-1] = 0; yylval.s = new std::string(yytext+1); return CPT_STRING; }
[0-9]+			{ yylval.i = atoi(yytext); return CPT_INTEGER; }
[-()<>=+*/%;{}.]	return *yytext;

[ \t\n]+		; /* ignore whitespace */

.			yyerror("Unknown character");
%%
