// $Id: Cwriter.cpp,v 1.5 2008/03/01 18:32:51 david Exp $ -*- c++ -*-
/*
 * $Log: Cwriter.cpp,v $
 * Revision 1.5  2008/03/01 18:32:51  david
 * Compact for CDK3 (first version).
 *
 * Revision 1.4  2007/03/17 19:19:36  david
 * *** empty log message ***
 *
 * Revision 1.3  2007/03/13 00:26:32  david
 * This version of the Compact compiler uses the new CDK2 library.
 * All nodes and visitors now use non-overloaded visitor hooks. The Compact
 * compiler now implements the abstract factory for scanners and compilers.
 *
 * Revision 1.2  2006/03/19 20:10:41  david
 * Corrected C generator bug: variables exist in a single program-wide scope.
 * Throw universal exception if variables are used improperly. 42.
 */

#include "Cwriter.h"
#include <string>

#include <cdk/nodes/Node.h>
#include <cdk/nodes/Data.h>
#include <cdk/nodes/Nil.h>
#include <cdk/nodes/Composite.h>
#include <cdk/nodes/Sequence.h>

#include <cdk/nodes/expressions/Integer.h>
#include <cdk/nodes/expressions/Double.h>
#include <cdk/nodes/expressions/String.h>
#include <cdk/nodes/expressions/Identifier.h>
#include <cdk/nodes/expressions/NEG.h>
#include <cdk/nodes/expressions/ADD.h>
#include <cdk/nodes/expressions/SUB.h>
#include <cdk/nodes/expressions/MUL.h>
#include <cdk/nodes/expressions/DIV.h>
#include <cdk/nodes/expressions/MOD.h>
#include <cdk/nodes/expressions/LT.h>
#include <cdk/nodes/expressions/GT.h>
#include <cdk/nodes/expressions/GE.h>
#include <cdk/nodes/expressions/LE.h>
#include <cdk/nodes/expressions/NE.h>
#include <cdk/nodes/expressions/EQ.h>

#include "nodes/ReadNode.h"
#include "nodes/PrintNode.h"
#include "nodes/AssignmentNode.h"
#include "nodes/WhileNode.h"
#include "nodes/IfNode.h"
#include "nodes/IfElseNode.h"
#include "nodes/ProgramNode.h"

#include "TypeValidator.h"

void Cwriter::processSequence(cdk::node::Sequence *const node, int lvl) {
  for (size_t i = 0; i < node->size(); i++) {
    node->node(i)->accept(this, lvl);
  }
}

void Cwriter::processInteger(cdk::node::expression::Integer *const node, int lvl) {
  os() << node->value();
}

void Cwriter::processDouble(cdk::node::expression::Double *const node, int lvl) {
  os() << node->value();
}

void Cwriter::processString(cdk::node::expression::String *const node, int lvl) {
  os() << std::string(lvl+2, ' ') << "printf(\"%s\\n\", \"" << node->value() << "\");\n";
}

void Cwriter::processIdentifier(cdk::node::expression::Identifier *const node, int lvl) {
  // DAVID: the horror!!
  const char *id = node->value().c_str();
  if (_symtab.find(id)) os() << id;
  else throw 42;  // life, the universe, and everything
}

void Cwriter::processNEG(cdk::node::expression::NEG *const node, int lvl) {
  os() << " - ";
  node->argument()->accept(this, lvl);
}

//protected:
inline void Cwriter::processBinaryExpression(cdk::node::expression::BinaryExpression *const node,
				int lvl, const char *tag) {
  node->left()->accept(this, lvl);
  os() << " " << tag << " ";
  node->right()->accept(this, lvl);
}

//public:
void Cwriter::processADD(cdk::node::expression::ADD *const node, int lvl) { processBinaryExpression(node, lvl, "+");  }
void Cwriter::processSUB(cdk::node::expression::SUB *const node, int lvl) { processBinaryExpression(node, lvl, "-");  }
void Cwriter::processMUL(cdk::node::expression::MUL *const node, int lvl) { processBinaryExpression(node, lvl, "*");  }
void Cwriter::processDIV(cdk::node::expression::DIV *const node, int lvl) { processBinaryExpression(node, lvl, "/");  }
void Cwriter::processMOD(cdk::node::expression::MOD *const node, int lvl) { processBinaryExpression(node, lvl, "%");  }
void Cwriter::processLT (cdk::node::expression::LT  *const node, int lvl) { processBinaryExpression(node, lvl, "<");  }
void Cwriter::processLE (cdk::node::expression::LE  *const node, int lvl) { processBinaryExpression(node, lvl, "<="); }
void Cwriter::processGE (cdk::node::expression::GE  *const node, int lvl) { processBinaryExpression(node, lvl, ">="); }
void Cwriter::processGT (cdk::node::expression::GT  *const node, int lvl) { processBinaryExpression(node, lvl, ">");  }
void Cwriter::processNE (cdk::node::expression::NE  *const node, int lvl) { processBinaryExpression(node, lvl, "!="); }
void Cwriter::processEQ (cdk::node::expression::EQ  *const node, int lvl) { processBinaryExpression(node, lvl, "=="); }

void Cwriter::processProgramNode(ProgramNode *const node, int lvl) {
  node->statements()->accept(this, lvl);
}

void Cwriter::processPrintNode(PrintNode *const node, int lvl) {
  os() << std::string(lvl+2, ' ') << "printf(\"%d\\n\", ";
  node->argument()->accept(this, lvl);
  os() << ");\n";
}

void Cwriter::processReadNode(ReadNode *const node, int lvl) {
  try {
    TypeValidator v(os(), _symtab);
    node->accept(&v, lvl);
  }
  catch (std::string s) {
    std::cerr << "FATAL: " << node->lineno() << ": " << s << std::endl;
    return;
  }

  // DAVID: horrible hack!
  const char *id = ((cdk::node::expression::Identifier*)node->argument())->value().c_str();
  if (_symtab.find(id))
    os() << std::string(lvl+2, ' ') << "scanf(\"%d\", &" << id << ");\n";
}

void Cwriter::processWhileNode(WhileNode *const node, int lvl) {
  try {
    TypeValidator v(os(), _symtab);
    node->accept(&v, lvl);
  }
  catch (std::string s) {
    std::cerr << "FATAL: " << node->lineno() << ": " << s << std::endl;
  }

  os() << std::string(lvl+2, ' ') << "while(";
  node->condition()->accept(this, lvl);
  os() << ") {\n";
  node->block()->accept(this, lvl+2);
  os() << std::string(lvl+2, ' ') << "}\n";
}

void Cwriter::processAssignmentNode(AssignmentNode *const node, int lvl) {
  // DAVID: horrible hack!
  const char *id = ((cdk::node::expression::Identifier*)node->lvalue())->value().c_str();
  if (_symtab.insert(id, new cdk::Symbol(0, id, 0))) {
    os() << std::string(lvl+2, ' ');
    os() << "int " << id << ";\n";
  }

  try {
    TypeValidator v(os(), _symtab);
    node->accept(&v, lvl);
  }
  catch (std::string s) {
    std::cerr << "FATAL: " << node->lineno() << ": " << s << std::endl;
    return;
  }

  os() << std::string(lvl+2, ' ');
  node->lvalue()->accept(this, lvl);
  os() << " = ";
  node->rvalue()->accept(this, lvl);
  os() << ";\n";
}

void Cwriter::processIfNode(IfNode *const node, int lvl) {
  try {
    TypeValidator v(os(), _symtab);
    node->accept(&v, lvl);
  }
  catch (std::string s) {
    std::cerr << "FATAL: " << node->lineno() << ": " << s << std::endl;
    return;
  }

  os() << std::string(lvl+2, ' ') << "if (";
  node->condition()->accept(this, lvl);
  os() << ") {\n";
  node->block()->accept(this, lvl+2);
  os() << std::string(lvl+2, ' ') << "}\n";
}

void Cwriter::processIfElseNode(IfElseNode *const node, int lvl) {
  try {
    TypeValidator v(os(), _symtab);
    node->accept(&v, lvl);
  }
  catch (std::string s) {
    std::cerr << "FATAL: " << node->lineno() << ": " << s << std::endl;
    return;
  }

  os() << std::string(lvl+2, ' ') << "if (";
  node->condition()->accept(this, lvl);
  os() << ") {\n";
  node->thenblock()->accept(this, lvl+2);
  os() << std::string(lvl+2, ' ') << "} else {\n";
  node->elseblock()->accept(this, lvl+2);
  os() << std::string(lvl+2, ' ') << "}\n";
}

//---------------------------------------------------------------------------
//     T H E    E N D
//---------------------------------------------------------------------------
