// $Id: Cwriter.h,v 1.5 2008/03/01 18:32:52 david Exp $ -*- c++ -*-
/*
 * $Log: Cwriter.h,v $
 * Revision 1.5  2008/03/01 18:32:52  david
 * Compact for CDK3 (first version).
 *
 * Revision 1.4  2007/03/17 19:19:36  david
 * *** empty log message ***
 *
 * Revision 1.3  2007/03/13 00:26:33  david
 * This version of the Compact compiler uses the new CDK2 library.
 * All nodes and visitors now use non-overloaded visitor hooks. The Compact
 * compiler now implements the abstract factory for scanners and compilers.
 *
 * Revision 1.2  2007/02/22 17:37:33  david
 * Fixed default parameter initialization error.
 *
 * Revision 1.1.1.1  2006/01/03 16:37:25  david
 */

#ifndef __COMPACT_SEMANTICS_CWRITER_H__
#define __COMPACT_SEMANTICS_CWRITER_H__

#include "semantics/SemanticProcessor.h"

#include <string>
#include <iostream>
#include <cdk/SymbolTable.h>

//!
//! Traverse syntax tree and generate the corresponding C code.
//!

class Cwriter : public virtual SemanticProcessor {
  cdk::SymbolTable &_symtab;

public:
  Cwriter(std::ostream &os, cdk::SymbolTable &symtab)
    : SemanticProcessor(os), _symtab(symtab) {}

public:
  ~Cwriter() { os().flush(); }

public:
  void processNode     (cdk::node::Node      *const node, int lvl) {}
  void processNil      (cdk::node::Nil       *const node, int lvl) {}
  void processData     (cdk::node::Data      *const node, int lvl) {}
  void processComposite(cdk::node::Composite *const node, int lvl) {}
  void processSequence (cdk::node::Sequence  *const node, int lvl);

public:
  template <typename T> void processSimple(cdk::node::expression::Simple<T> *const node, int lvl) {}
  void processInteger   (cdk::node::expression::Integer    *const node, int lvl);
  void processDouble    (cdk::node::expression::Double     *const node, int lvl);
  void processString    (cdk::node::expression::String     *const node, int lvl);
  void processIdentifier(cdk::node::expression::Identifier *const node, int lvl);

protected:
  void processUnaryExpression(cdk::node::expression::UnaryExpression *const node, int lvl, const char *tag);

public:
  void processUnaryExpression(cdk::node::expression::UnaryExpression *const node, int lvl) {}
  void processNEG(cdk::node::expression::NEG *const node, int lvl);

protected:
  void processBinaryExpression(cdk::node::expression::BinaryExpression *const node, int lvl, const char *tag);

public:
  void processBinaryExpression(cdk::node::expression::BinaryExpression *const node, int lvl) {}
  void processADD(cdk::node::expression::ADD *const node, int lvl);
  void processSUB(cdk::node::expression::SUB *const node, int lvl);
  void processMUL(cdk::node::expression::MUL *const node, int lvl);
  void processDIV(cdk::node::expression::DIV *const node, int lvl);
  void processMOD(cdk::node::expression::MOD *const node, int lvl);
  void processLT (cdk::node::expression::LT  *const node, int lvl);
  void processLE (cdk::node::expression::LE  *const node, int lvl);
  void processGE (cdk::node::expression::GE  *const node, int lvl);
  void processGT (cdk::node::expression::GT  *const node, int lvl);
  void processNE (cdk::node::expression::NE  *const node, int lvl);
  void processEQ (cdk::node::expression::EQ  *const node, int lvl);

public:
  void processProgramNode   (ProgramNode    *const node, int lvl);
  void processPrintNode     (PrintNode      *const node, int lvl);
  void processReadNode      (ReadNode       *const node, int lvl);
  void processWhileNode     (WhileNode      *const node, int lvl);
  void processAssignmentNode(AssignmentNode *const node, int lvl);
  void processIfNode        (IfNode         *const node, int lvl);
  void processIfElseNode    (IfElseNode     *const node, int lvl);

};

#endif
