// $Id: Interpreter.cpp,v 1.5 2008/03/01 18:32:52 david Exp $ -*- c++ -*-
/*
 * $Log: Interpreter.cpp,v $
 * Revision 1.5  2008/03/01 18:32:52  david
 * Compact for CDK3 (first version).
 *
 * Revision 1.4  2007/03/17 19:19:36  david
 * *** empty log message ***
 *
 * Revision 1.3  2007/03/13 00:26:33  david
 * This version of the Compact compiler uses the new CDK2 library.
 * All nodes and visitors now use non-overloaded visitor hooks. The Compact
 * compiler now implements the abstract factory for scanners and compilers.
 *
 * Revision 1.2  2006/03/19 21:46:52  david
 * Miscellaneous corrections to i/o streams.
 */

#include "Interpreter.h"
#include <string>

#include <cdk/nodes/Node.h>
#include <cdk/nodes/Data.h>
#include <cdk/nodes/Nil.h>
#include <cdk/nodes/Composite.h>
#include <cdk/nodes/Sequence.h>

#include <cdk/nodes/expressions/Integer.h>
#include <cdk/nodes/expressions/Double.h>
#include <cdk/nodes/expressions/String.h>
#include <cdk/nodes/expressions/Identifier.h>
#include <cdk/nodes/expressions/NEG.h>
#include <cdk/nodes/expressions/ADD.h>
#include <cdk/nodes/expressions/SUB.h>
#include <cdk/nodes/expressions/MUL.h>
#include <cdk/nodes/expressions/DIV.h>
#include <cdk/nodes/expressions/MOD.h>
#include <cdk/nodes/expressions/LT.h>
#include <cdk/nodes/expressions/GT.h>
#include <cdk/nodes/expressions/GE.h>
#include <cdk/nodes/expressions/LE.h>
#include <cdk/nodes/expressions/NE.h>
#include <cdk/nodes/expressions/EQ.h>

#include "nodes/ReadNode.h"
#include "nodes/PrintNode.h"
#include "nodes/AssignmentNode.h"
#include "nodes/WhileNode.h"
#include "nodes/IfNode.h"
#include "nodes/IfElseNode.h"
#include "nodes/ProgramNode.h"

void Interpreter::processSequence(cdk::node::Sequence *const node, int lvl) {
  for (size_t i = 0; i < node->size(); i++)
    node->node(i)->accept(this, lvl+2);
}

//public:
void Interpreter::processInteger(cdk::node::expression::Integer *const node, int lvl) {
  _stack.push(node->value());
}

void Interpreter::processDouble(cdk::node::expression::Double *const node, int lvl) {}
void Interpreter::processString(cdk::node::expression::String *const node, int lvl) {
  os() << node->value() << std::endl;
}

void Interpreter::processIdentifier(cdk::node::expression::Identifier *const node, int lvl) {
  // DAVID: the horror!!
  const char *id = node->value().c_str();
  cdk::Symbol *symbol = _symtab.find(id);
  _stack.push(symbol->value());
}

//public:
void Interpreter::processNEG(cdk::node::expression::NEG *const node, int lvl) {
  node->argument()->accept(this, lvl);
  _stack.push(-_stack.pop());
}

//public:
void Interpreter::processADD(cdk::node::expression::ADD *const node, int lvl) {
  node->right()->accept(this, lvl);
  node->left()->accept(this, lvl);
  _stack.push(_stack.pop() + _stack.pop());
}
void Interpreter::processSUB(cdk::node::expression::SUB *const node, int lvl) {
  node->right()->accept(this, lvl);
  node->left()->accept(this, lvl);
  _stack.push(_stack.pop() - _stack.pop());
}
void Interpreter::processMUL(cdk::node::expression::MUL *const node, int lvl) {
  node->right()->accept(this, lvl);
  node->left()->accept(this, lvl);
  _stack.push(_stack.pop() * _stack.pop());
}
void Interpreter::processDIV(cdk::node::expression::DIV *const node, int lvl) {
  node->right()->accept(this, lvl);
  node->left()->accept(this, lvl);
  _stack.push(_stack.pop() / _stack.pop());
}
void Interpreter::processMOD(cdk::node::expression::MOD *const node, int lvl) {
  node->right()->accept(this, lvl);
  node->left()->accept(this, lvl);
  _stack.push(_stack.pop() % _stack.pop());
}
void Interpreter::processLT(cdk::node::expression::LT  *const node, int lvl) {
  node->right()->accept(this, lvl);
  node->left()->accept(this, lvl);
  _stack.push(_stack.pop() < _stack.pop());
}
void Interpreter::processLE(cdk::node::expression::LE  *const node, int lvl) {
  node->right()->accept(this, lvl);
  node->left()->accept(this, lvl);
  _stack.push(_stack.pop() <= _stack.pop());
}
void Interpreter::processGE(cdk::node::expression::GE  *const node, int lvl) {
  node->right()->accept(this, lvl);
  node->left()->accept(this, lvl);
  _stack.push(_stack.pop() >= _stack.pop());
}
void Interpreter::processGT(cdk::node::expression::GT  *const node, int lvl) {
  node->right()->accept(this, lvl);
  node->left()->accept(this, lvl);
  _stack.push(_stack.pop() > _stack.pop());
}
void Interpreter::processNE(cdk::node::expression::NE  *const node, int lvl) {
  node->right()->accept(this, lvl);
  node->left()->accept(this, lvl);
  _stack.push(_stack.pop() != _stack.pop());
}
void Interpreter::processEQ(cdk::node::expression::EQ  *const node, int lvl) {
  node->right()->accept(this, lvl);
  node->left()->accept(this, lvl);
  _stack.push(_stack.pop() == _stack.pop());
}

void Interpreter::processProgramNode(ProgramNode *const node, int lvl) {
  node->statements()->accept(this, lvl);
}

void Interpreter::processPrintNode(PrintNode *const node, int lvl) {
  node->argument()->accept(this, lvl);
  os() << _stack.pop() << std::endl;
}

void Interpreter::processReadNode(ReadNode *const node, int lvl) {
  // DAVID: horrible hack!
  const char  *id  = ((cdk::node::expression::Identifier*)node->argument())->value().c_str();
  cdk::Symbol *symbol = _symtab.find(id);
  if (symbol != NULL) {
    int val;
    std::cin >> val;
    symbol->value(val);
  }
}

void Interpreter::processWhileNode(WhileNode *const node, int lvl) {
  node->condition()->accept(this, lvl);
  while (_stack.pop()) {
    node->block()->accept(this, lvl);
    node->condition()->accept(this, lvl);
  }
}

void Interpreter::processAssignmentNode(AssignmentNode *const node, int lvl) {
  // DAVID: horrible hack!
  const char *id = ((cdk::node::expression::Identifier*)node->lvalue())->value().c_str();
  cdk::Symbol *symbol = _symtab.find(id);
  if (symbol == NULL) {
    symbol = new cdk::Symbol(0, id, 0);
    _symtab.insert(id, symbol);
    symbol = _symtab.find(id);
  }
  node->rvalue()->accept(this, lvl);
  symbol->value(_stack.pop());
}

void Interpreter::processIfNode(IfNode *const node, int lvl) {
  node->condition()->accept(this, lvl);
  if (_stack.pop()) node->block()->accept(this, lvl);
}

void Interpreter::processIfElseNode(IfElseNode *const node, int lvl) {
  node->condition()->accept(this, lvl);
  if (_stack.pop()) node->thenblock()->accept(this, lvl);
  else              node->elseblock()->accept(this, lvl);
}

//---------------------------------------------------------------------------
//     T H E    E N D
//---------------------------------------------------------------------------
