// $Id: PFevaluator.cpp,v 1.1 2007/03/14 01:20:28 david Exp $
/*
 * $Log: PFevaluator.cpp,v $
 * Revision 1.1  2007/03/14 01:20:28  david
 * Changed compiler implementation to use evaluation strategy objects
 * instead of functions. These strategies are selected according to the
 * target selected from the command line options or from clues such as
 * the output file extension.
 *
 */

//---------------------------------------------------------------------------

#include <cdk/nodes/Node.h>
#include <cdk/SymbolTable.h>
#include <cdk/Compiler.h>
#include <cdk/generators/ix86.h>
#include "semantics/PFevaluator.h"
#include "semantics/PFwriter.h"

//---------------------------------------------------------------------------

/**
 * Postfix for ix86.
 * @var create and register an evaluator for ASM targets.
 */
PFevaluator PFevaluator::_thisEvaluator("asm");

//---------------------------------------------------------------------------

bool PFevaluator::evaluate(cdk::Compiler *compiler) {
  bool debug = false; // HACK! (should be passed as arg)

  std::ostream &os = compiler->ostream();

  // this symbol table will be used to check identifiers
  cdk::SymbolTable symtab;
  cdk::generator::ix86 pf(os, debug);

  // we must open the outer function
  //
  pf.TEXT();
  pf.ALIGN();
  pf.GLOBL("_main", pf.FUNC());
  pf.LABEL("_main");
  pf.ENTER(0);

  // now the rest of the nasm code will be generated
  // identifiers will be checked during code generation
  //
  PFwriter pfwriter(os, symtab, pf);
  compiler->syntax()->accept(&pfwriter, 0);

  // We must close the outer function
  // the output stream must be the same used for the two processors
  //
  pf.INT(0);
  pf.POP();
  pf.LEAVE();
  pf.RET();

  // these are just a few library function imports
  //
  pf.EXTRN("readi");
  pf.EXTRN("printi");
  pf.EXTRN("prints");
  pf.EXTRN("println");

  return true;
}

//---------------------------------------------------------------------------
//     T H E    E N D
//---------------------------------------------------------------------------
