// $Id: PFwriter.cpp,v 1.5 2008/03/01 18:32:52 david Exp $ -*- c++ -*-
/*
 * $Log: PFwriter.cpp,v $
 * Revision 1.5  2008/03/01 18:32:52  david
 * Compact for CDK3 (first version).
 *
 * Revision 1.4  2007/03/17 19:19:37  david
 * *** empty log message ***
 *
 * Revision 1.3  2007/03/13 00:26:34  david
 * This version of the Compact compiler uses the new CDK2 library.
 * All nodes and visitors now use non-overloaded visitor hooks. The Compact
 * compiler now implements the abstract factory for scanners and compilers.
 */

#include "PFwriter.h"
#include <string>
#include <sstream>

//---------------------------------------------------------------------------
//     THESE ARE PREDEFINED NODES, AVAILABLE FROM THE CDK
//---------------------------------------------------------------------------

#include <cdk/nodes/Node.h>
#include <cdk/nodes/Data.h>
#include <cdk/nodes/Nil.h>
#include <cdk/nodes/Composite.h>
#include <cdk/nodes/Sequence.h>

#include <cdk/nodes/expressions/Integer.h>
#include <cdk/nodes/expressions/Double.h>
#include <cdk/nodes/expressions/String.h>
#include <cdk/nodes/expressions/Identifier.h>
#include <cdk/nodes/expressions/NEG.h>
#include <cdk/nodes/expressions/ADD.h>
#include <cdk/nodes/expressions/SUB.h>
#include <cdk/nodes/expressions/MUL.h>
#include <cdk/nodes/expressions/DIV.h>
#include <cdk/nodes/expressions/MOD.h>
#include <cdk/nodes/expressions/LT.h>
#include <cdk/nodes/expressions/GT.h>
#include <cdk/nodes/expressions/GE.h>
#include <cdk/nodes/expressions/LE.h>
#include <cdk/nodes/expressions/NE.h>
#include <cdk/nodes/expressions/EQ.h>

//---------------------------------------------------------------------------
//     THESE ARE THE NODES DEFINED SPECIFICALLY FOR THIS APPLICATION
//---------------------------------------------------------------------------

#include "nodes/ReadNode.h"
#include "nodes/PrintNode.h"
#include "nodes/AssignmentNode.h"
#include "nodes/WhileNode.h"
#include "nodes/IfNode.h"
#include "nodes/IfElseNode.h"
#include "nodes/ProgramNode.h"

//---------------------------------------------------------------------------
//     THIS IS THE EVALUATOR'S DEFINITION
//---------------------------------------------------------------------------

//private:
std::string PFwriter::mklbl(int lbl) {
  std::ostringstream oss;
  if (lbl < 0) oss << ".L" << -lbl;
  else         oss << "_L" <<  lbl;
  return oss.str();
}

//---------------------------------------------------------------------------

void PFwriter::processSequence(cdk::node::Sequence *const node, int lvl) {
  for (size_t i = 0; i < node->size(); i++) {
    node->node(i)->accept(this, lvl);
  }
}

//---------------------------------------------------------------------------

void PFwriter::processInteger(cdk::node::expression::Integer *const node, int lvl) {
  _pf.INT(node->value());                // push an integer
}

void PFwriter::processDouble(cdk::node::expression::Double *const node, int lvl) {}
void PFwriter::processString(cdk::node::expression::String *const node, int lvl) {
  int lbl1;

  /* generate the string */
  _pf.RODATA();                      // strings are DATA readonly
  _pf.ALIGN();                       // make sure we are aligned
  _pf.LABEL(mklbl(lbl1 = ++_lbl));   // give the string a name
  _pf.STR(node->value());            // output string characters

  /* make the call */
  _pf.TEXT();                        // return to the TEXT segment
  _pf.ADDR(mklbl(lbl1));             // the string to be printed
  _pf.CALL("prints");                // call the print rotine
  _pf.CALL("println");               // print a newline
  _pf.TRASH(4);                      // remove the string label
}

//---------------------------------------------------------------------------

void PFwriter::processIdentifier(cdk::node::expression::Identifier *const node, int lvl) {
  // DAVID: the horror!!
  const char *id = node->value().c_str();
  if (_symtab.find(id)) {
    _pf.ADDR(id);
    _pf.LOAD();
  }
}

//---------------------------------------------------------------------------

void PFwriter::processNEG(cdk::node::expression::NEG *const node, int lvl) {
  node->argument()->accept(this, lvl);    // determine the value
  _pf.NEG();                              // 2-complement
}

//---------------------------------------------------------------------------

//public:
void PFwriter::processADD(cdk::node::expression::ADD *const node, int lvl) {
  node->left()->accept(this, lvl);
  node->right()->accept(this, lvl);
  _pf.ADD();
}
void PFwriter::processSUB(cdk::node::expression::SUB *const node, int lvl) {
  node->left()->accept(this, lvl);
  node->right()->accept(this, lvl);
  _pf.SUB();
}
void PFwriter::processMUL(cdk::node::expression::MUL *const node, int lvl) {
  node->left()->accept(this, lvl);
  node->right()->accept(this, lvl);
  _pf.MUL();
}
void PFwriter::processDIV(cdk::node::expression::DIV *const node, int lvl) {
  node->left()->accept(this, lvl);
  node->right()->accept(this, lvl);
  _pf.DIV();
}
void PFwriter::processMOD(cdk::node::expression::MOD *const node, int lvl) {
  node->left()->accept(this, lvl);
  node->right()->accept(this, lvl);
  _pf.MOD();
}
void PFwriter::processLT(cdk::node::expression::LT  *const node, int lvl) {
  node->left()->accept(this, lvl);
  node->right()->accept(this, lvl);
  _pf.LT();
}
void PFwriter::processLE(cdk::node::expression::LE  *const node, int lvl) {
  node->left()->accept(this, lvl);
  node->right()->accept(this, lvl);
  _pf.LE();
}
void PFwriter::processGE(cdk::node::expression::GE  *const node, int lvl) {
  node->left()->accept(this, lvl);
  node->right()->accept(this, lvl);
  _pf.GE();
}
void PFwriter::processGT(cdk::node::expression::GT  *const node, int lvl) {
  node->left()->accept(this, lvl);
  node->right()->accept(this, lvl);
  _pf.GT();
}
void PFwriter::processNE(cdk::node::expression::NE  *const node, int lvl) {
  node->left()->accept(this, lvl);
  node->right()->accept(this, lvl);
  _pf.NE();
}
void PFwriter::processEQ(cdk::node::expression::EQ  *const node, int lvl) {
  node->left()->accept(this, lvl);
  node->right()->accept(this, lvl);
  _pf.EQ();
}

//---------------------------------------------------------------------------

void PFwriter::processProgramNode(ProgramNode *const node, int lvl) {
  node->statements()->accept(this, lvl);
}

//---------------------------------------------------------------------------

void PFwriter::processPrintNode(PrintNode *const node, int lvl) {
  node->argument()->accept(this, lvl);     // determine the value to print
  _pf.CALL("printi");                      // call to the print function
  _pf.CALL("println");                     // print a newline
  _pf.TRASH(4);                            // delete the printed value
}

//---------------------------------------------------------------------------

void PFwriter::processReadNode(ReadNode *const node, int lvl) {
  // DAVID: horrible hack!
  const char *id = ((cdk::node::expression::Identifier*)node->argument())->value().c_str();
  if (_symtab.find(id)) {
    _pf.CALL("readi");
    _pf.PUSH();
    _pf.ADDR(id);
    _pf.STORE();
  }
}

//---------------------------------------------------------------------------

void PFwriter::processWhileNode(WhileNode *const node, int lvl) {
  int lbl1, lbl2;
  _pf.LABEL(mklbl(lbl1 = ++_lbl));
  node->condition()->accept(this, lvl);
  _pf.JZ(mklbl(lbl2 = ++_lbl));
  node->block()->accept(this, lvl+2);
  _pf.JMP(mklbl(lbl1));
  _pf.LABEL(mklbl(lbl2));
}

//---------------------------------------------------------------------------

void PFwriter::processAssignmentNode(AssignmentNode *const node, int lvl) {
  // DAVID: horrible hack!
  const char *id = ((cdk::node::expression::Identifier*)node->lvalue())->value().c_str();
  if (!_symtab.find(id)) {
    _pf.DATA();                                 // variables are DATA
    _pf.ALIGN();                                // make sure we are aligned
    _pf.LABEL(id);                              // name variable location
    _pf.CONST(0);                               // initialize it to 0 (zero)
    _pf.TEXT();                                 // return to the TEXT segment
    _symtab.insert(id, new cdk::Symbol(0, id, 0));   // put in the symbol table
  }
  node->rvalue()->accept(this, lvl);      // determine the new value
  _pf.ADDR(id);                           // where to store the value
  _pf.STORE();                            // store the value at address
}

//---------------------------------------------------------------------------

void PFwriter::processIfNode(IfNode *const node, int lvl) {
  int lbl1;
  node->condition()->accept(this, lvl);
  _pf.JZ(mklbl(lbl1 = ++_lbl));
  node->block()->accept(this, lvl+2);
  _pf.LABEL(mklbl(lbl1));
}

//---------------------------------------------------------------------------

void PFwriter::processIfElseNode(IfElseNode *const node, int lvl) {
  int lbl1, lbl2;
  node->condition()->accept(this, lvl);
  _pf.JZ(mklbl(lbl1 = ++_lbl));
  node->thenblock()->accept(this, lvl+2);
  _pf.JMP(mklbl(lbl2 = ++_lbl));
  _pf.LABEL(mklbl(lbl1));
  node->elseblock()->accept(this, lvl+2);
  _pf.LABEL(mklbl(lbl1 = lbl2));
}

//---------------------------------------------------------------------------
//     T H E    E N D
//---------------------------------------------------------------------------
