// $Id: SemanticProcessor.h,v 1.5 2008/03/01 18:32:52 david Exp $ -*- c++ -*-
/*
 * $Log: SemanticProcessor.h,v $
 * Revision 1.5  2008/03/01 18:32:52  david
 * Compact for CDK3 (first version).
 *
 * Revision 1.4  2007/03/17 19:19:38  david
 * *** empty log message ***
 *
 * Revision 1.3  2007/03/13 00:26:34  david
 * This version of the Compact compiler uses the new CDK2 library.
 * All nodes and visitors now use non-overloaded visitor hooks. The Compact
 * compiler now implements the abstract factory for scanners and compilers.
 */

#ifndef __COMPACT_SEMANTICS_SEMANTICPROCESSOR_H__
#define __COMPACT_SEMANTICS_SEMANTICPROCESSOR_H__

#include <string>
#include <iostream>

namespace cdk {
  namespace node {
    class Node; class Nil; class Data; class Composite; class Sequence;
    namespace expression {
      template <class T> class Simple;
      class Double; class Integer; class String; class Identifier;
      class UnaryExpression;
      class NEG;
      class BinaryExpression;
      class ADD; class SUB; class MUL; class DIV; class MOD; class LT;
      class LE; class GE; class GT; class EQ; class NE;
    } // namespace expression
  } // namespace node
} // namespace cdk

class ProgramNode;
class PrintNode;
class ReadNode;
class WhileNode;
class IfNode;
class IfElseNode;
class AssignmentNode;

//!
//! Print nodes as XML elements to the output stream.
//!
class SemanticProcessor {
  //! The output stream
  std::ostream &_os;
  
protected:
  SemanticProcessor(std::ostream &os = std::cout) : _os(os) {}
  inline std::ostream &os() { return _os; }
  
public:
  virtual ~SemanticProcessor() {}

public:
  virtual void processNode     (cdk::node::Node      *const node, int lvl) = 0;
  virtual void processNil      (cdk::node::Nil       *const node, int lvl) = 0;
  virtual void processData     (cdk::node::Data      *const node, int lvl) = 0;
  virtual void processComposite(cdk::node::Composite *const node, int lvl) = 0;
  virtual void processSequence (cdk::node::Sequence  *const node, int lvl) = 0;

public:
  template <typename T> void processSimple(cdk::node::expression::Simple<T> *const node, int lvl) { throw 42; }
  virtual void processInteger   (cdk::node::expression::Integer    *const node, int lvl) = 0;
  virtual void processDouble    (cdk::node::expression::Double     *const node, int lvl) = 0;
  virtual void processString    (cdk::node::expression::String     *const node, int lvl) = 0;
  virtual void processIdentifier(cdk::node::expression::Identifier *const node, int lvl) = 0;

public:
  virtual void processUnaryExpression(cdk::node::expression::UnaryExpression *const node, int lvl) = 0;
  virtual void processNEG(cdk::node::expression::NEG *const node, int lvl) = 0;

public:
  virtual void processBinaryExpression(cdk::node::expression::BinaryExpression *const node, int lvl) = 0;
  virtual void processADD(cdk::node::expression::ADD *const node, int lvl) = 0;
  virtual void processSUB(cdk::node::expression::SUB *const node, int lvl) = 0;
  virtual void processMUL(cdk::node::expression::MUL *const node, int lvl) = 0;
  virtual void processDIV(cdk::node::expression::DIV *const node, int lvl) = 0;
  virtual void processMOD(cdk::node::expression::MOD *const node, int lvl) = 0;
  virtual void processLT (cdk::node::expression::LT  *const node, int lvl) = 0;
  virtual void processLE (cdk::node::expression::LE  *const node, int lvl) = 0;
  virtual void processGE (cdk::node::expression::GE  *const node, int lvl) = 0;
  virtual void processGT (cdk::node::expression::GT  *const node, int lvl) = 0;
  virtual void processNE (cdk::node::expression::NE  *const node, int lvl) = 0;
  virtual void processEQ (cdk::node::expression::EQ  *const node, int lvl) = 0;

public:
  virtual void processProgramNode   (ProgramNode    *const node, int lvl) = 0;
  virtual void processPrintNode     (PrintNode      *const node, int lvl) = 0;
  virtual void processReadNode      (ReadNode       *const node, int lvl) = 0;
  virtual void processWhileNode     (WhileNode      *const node, int lvl) = 0;
  virtual void processAssignmentNode(AssignmentNode *const node, int lvl) = 0;
  virtual void processIfNode        (IfNode         *const node, int lvl) = 0;
  virtual void processIfElseNode    (IfElseNode     *const node, int lvl) = 0;

};

#endif
