// $Id: XMLwriter.cpp,v 1.4 2008/03/01 18:32:52 david Exp $ -*- c++ -*-
/*
 * $Log: XMLwriter.cpp,v $
 * Revision 1.4  2008/03/01 18:32:52  david
 * Compact for CDK3 (first version).
 *
 * Revision 1.3  2007/03/17 19:19:39  david
 * *** empty log message ***
 *
 * Revision 1.2  2007/03/13 00:26:35  david
 * This version of the Compact compiler uses the new CDK2 library.
 * All nodes and visitors now use non-overloaded visitor hooks. The Compact
 * compiler now implements the abstract factory for scanners and compilers.
 */

#include "XMLwriter.h"
#include <string>

#include <cdk/nodes/Node.h>
#include <cdk/nodes/Data.h>
#include <cdk/nodes/Nil.h>
#include <cdk/nodes/Composite.h>
#include <cdk/nodes/Sequence.h>

#include <cdk/nodes/expressions/Integer.h>
#include <cdk/nodes/expressions/Double.h>
#include <cdk/nodes/expressions/String.h>
#include <cdk/nodes/expressions/Identifier.h>
#include <cdk/nodes/expressions/NEG.h>
#include <cdk/nodes/expressions/ADD.h>
#include <cdk/nodes/expressions/SUB.h>
#include <cdk/nodes/expressions/MUL.h>
#include <cdk/nodes/expressions/DIV.h>
#include <cdk/nodes/expressions/MOD.h>
#include <cdk/nodes/expressions/LT.h>
#include <cdk/nodes/expressions/GT.h>
#include <cdk/nodes/expressions/GE.h>
#include <cdk/nodes/expressions/LE.h>
#include <cdk/nodes/expressions/NE.h>
#include <cdk/nodes/expressions/EQ.h>

#include "nodes/ReadNode.h"
#include "nodes/PrintNode.h"
#include "nodes/AssignmentNode.h"
#include "nodes/WhileNode.h"
#include "nodes/IfNode.h"
#include "nodes/IfElseNode.h"
#include "nodes/ProgramNode.h"

void XMLwriter::processData(cdk::node::Data *const node, int lvl) {
  os() << std::string(lvl, ' ') << "<data size='" << node->size() << "'>";
  for (size_t i = 0; i < node->size(); i++)
    os() << ((unsigned char*)node->data())[i];
  os() << "</data>" << std::endl;
}

void XMLwriter::processComposite(cdk::node::Composite *const node, int lvl) {
  for (size_t i = 0; i < node->size(); i++)
    node->at(i)->accept(this, lvl+2);
}

void XMLwriter::processSequence(cdk::node::Sequence *const node, int lvl) {
  os() << std::string(lvl, ' ') << "<sequence size='"
       << node->size() << "'>" << std::endl;
  for (size_t i = 0; i < node->size(); i++) {
    node->node(i)->accept(this, lvl+2);
  }
  os() << std::string(lvl, ' ') << "</sequence>" << std::endl;
}

//public:
void XMLwriter::processInteger(cdk::node::expression::Integer *const node, int lvl) {
  processSimple(node, lvl, "integer");
}

void XMLwriter::processDouble(cdk::node::expression::Double *const node, int lvl) {
  processSimple(node, lvl, "double");
}

void XMLwriter::processString(cdk::node::expression::String *const node, int lvl) {
  processSimple(node, lvl, "string");
}

void XMLwriter::processIdentifier(cdk::node::expression::Identifier *const node, int lvl) {
  processSimple(node, lvl, "identifier");
}

//protected:
inline void XMLwriter::processUnaryExpression(cdk::node::expression::UnaryExpression *const node,
				int lvl, const char *tag) {
  os() << std::string(lvl, ' ') << "<" << tag << ">" << std::endl;
  node->argument()->accept(this, lvl+2);
  os() << std::string(lvl, ' ') << "</" << tag << ">" << std::endl;
}

//public:
void XMLwriter::processNEG(cdk::node::expression::NEG *const node, int lvl) { processUnaryExpression(node, lvl, "NEG"); }

//protected:
inline void XMLwriter::processBinaryExpression(cdk::node::expression::BinaryExpression *const node,
				int lvl, const char *tag) {
  os() << std::string(lvl, ' ') << "<" << tag << ">" << std::endl;
  node->left()->accept(this, lvl+2);
  node->right()->accept(this, lvl+2);
  os() << std::string(lvl, ' ') << "</" << tag << ">" << std::endl;
}

//public:
void XMLwriter::processADD(cdk::node::expression::ADD *const node, int lvl) { processBinaryExpression(node, lvl, "ADD"); }
void XMLwriter::processSUB(cdk::node::expression::SUB *const node, int lvl) { processBinaryExpression(node, lvl, "SUB"); }
void XMLwriter::processMUL(cdk::node::expression::MUL *const node, int lvl) { processBinaryExpression(node, lvl, "MUL"); }
void XMLwriter::processDIV(cdk::node::expression::DIV *const node, int lvl) { processBinaryExpression(node, lvl, "DIV"); }
void XMLwriter::processMOD(cdk::node::expression::MOD *const node, int lvl) { processBinaryExpression(node, lvl, "MOD"); }
void XMLwriter::processLT (cdk::node::expression::LT  *const node, int lvl) { processBinaryExpression(node, lvl, "LT");  }
void XMLwriter::processLE (cdk::node::expression::LE  *const node, int lvl) { processBinaryExpression(node, lvl, "LE");  }
void XMLwriter::processGE (cdk::node::expression::GE  *const node, int lvl) { processBinaryExpression(node, lvl, "GE");  }
void XMLwriter::processGT (cdk::node::expression::GT  *const node, int lvl) { processBinaryExpression(node, lvl, "GT");  }
void XMLwriter::processNE (cdk::node::expression::NE  *const node, int lvl) { processBinaryExpression(node, lvl, "NE");  }
void XMLwriter::processEQ (cdk::node::expression::EQ  *const node, int lvl) { processBinaryExpression(node, lvl, "EQ");  }

void XMLwriter::processProgramNode(ProgramNode *const node, int lvl) {
  os() << std::string(lvl,   ' ') << "<program>"     << std::endl;
  os() << std::string(lvl+2, ' ') << "<statements>"  << std::endl;
  node->statements()->accept(this, lvl+4);
  os() << std::string(lvl+2, ' ') << "</statements>" << std::endl;
  os() << std::string(lvl,   ' ') << "</program>"    << std::endl;
}

void XMLwriter::processPrintNode(PrintNode *const node, int lvl) {
  os() << std::string(lvl, ' ') << "<print>"  << std::endl;
  node->argument()->accept(this, lvl+2);
  os() << std::string(lvl, ' ') << "</print>" << std::endl;
}

void XMLwriter::processReadNode(ReadNode *const node, int lvl) {
  os() << std::string(lvl, ' ') << "<read>"  << std::endl;
  node->argument()->accept(this, lvl+2);
  os() << std::string(lvl, ' ') << "</read>" << std::endl;
}

void XMLwriter::processWhileNode(WhileNode *const node, int lvl) {
  os() << std::string(lvl,   ' ') << "<while>"      << std::endl;
  os() << std::string(lvl+2, ' ') << "<condition>"  << std::endl;
  node->condition()->accept(this, lvl+4);
  os() << std::string(lvl+2, ' ') << "</condition>" << std::endl;
  os() << std::string(lvl+2, ' ') << "<block>"      << std::endl;
  node->block()->accept(this, lvl+4);
  os() << std::string(lvl+2, ' ') << "</block>"     << std::endl;
  os() << std::string(lvl,   ' ') << "</while>"     << std::endl;
}

void XMLwriter::processAssignmentNode(AssignmentNode *const node, int lvl) {
  os() << std::string(lvl,   ' ') << "<assignment>"  << std::endl;
  os() << std::string(lvl+2, ' ') << "<lvalue>"      << std::endl;
  node->lvalue()->accept(this, lvl+4);
  os() << std::string(lvl+2, ' ') << "</lvalue>"     << std::endl;
  os() << std::string(lvl+2, ' ') << "<rvalue>"      << std::endl;
  node->rvalue()->accept(this, lvl+4);
  os() << std::string(lvl+2, ' ') << "</rvalue>"     << std::endl;
  os() << std::string(lvl,   ' ') << "</assignment>" << std::endl;
}

void XMLwriter::processIfNode(IfNode *const node, int lvl) {
  os() << std::string(lvl,   ' ') << "<if>"         << std::endl;
  os() << std::string(lvl+2, ' ') << "<condition>"  << std::endl;
  node->condition()->accept(this, lvl+4);
  os() << std::string(lvl+2, ' ') << "</condition>" << std::endl;
  os() << std::string(lvl+2, ' ') << "<then>"       << std::endl;
  node->block()->accept(this, lvl+4);
  os() << std::string(lvl+2, ' ') << "</then>"      << std::endl;
  os() << std::string(lvl,   ' ') << "</if>"        << std::endl;
}

void XMLwriter::processIfElseNode(IfElseNode *const node, int lvl) {
  os() << std::string(lvl,   ' ') << "<if>"         << std::endl;
  os() << std::string(lvl+2, ' ') << "<condition>"  << std::endl;
  node->condition()->accept(this, lvl+4);
  os() << std::string(lvl+2, ' ') << "</condition>" << std::endl;
  os() << std::string(lvl+2, ' ') << "<then>"       << std::endl;
  node->thenblock()->accept(this, lvl+4);
  os() << std::string(lvl+2, ' ') << "</then>"      << std::endl;
  os() << std::string(lvl+2, ' ') << "<else>"       << std::endl;
  node->elseblock()->accept(this, lvl+4);
  os() << std::string(lvl+2, ' ') << "</else>"      << std::endl;
  os() << std::string(lvl,   ' ') << "</if>"        << std::endl;
}

//---------------------------------------------------------------------------
//     T H E    E N D
//---------------------------------------------------------------------------
